@@ -6,7 +6,7 @@ use Mouse;
 use Lemonldap::NG::Common::Conf::Constants;
 use Lemonldap::NG::Common::Conf::ReConstants;
 
-our $VERSION = '2.0.5';
+our $VERSION = '2.0.6';
 
 extends 'Lemonldap::NG::Common::Conf::AccessLib';
 
@@ -662,7 +662,7 @@ sub _scanCatsAndApps {
         }
         else {
             $item->{title} = $apps->{$cat}->{options}->{name};
-            $item->{type} = $apps->{$cat}->{type} = 'menuApp';
+            $item->{type}  = $apps->{$cat}->{type} = 'menuApp';
             foreach my $o (
                 grep { not /^name$/ }
                 keys %{ $apps->{$cat}->{options} }
@@ -698,6 +698,26 @@ sub combModules {
     return $self->sendJSONresponse( $req, $res );
 }
 
+sub sfExtra {
+    my ( $self, $req, $key ) = @_;
+    return $self->sendError( $req, 'Subkeys forbidden for sfExtra', 400 )
+      if ($key);
+    my $val = $self->getConfKey( $req, 'sfExtra' ) // {};
+    my $res = [];
+    foreach my $mod ( keys %$val ) {
+        my $tmp;
+        $tmp->{title}      = $mod;
+        $tmp->{id}         = "sfExtra/$mod";
+        $tmp->{type}       = 'sfExtra';
+        $tmp->{data}->{$_} = $val->{$mod}->{$_}
+          foreach (qw(type rule logo label));
+        my $over = $val->{$mod}->{over} // {};
+        $tmp->{data}->{over} = [ map { [ $_, $over->{$_} ] } keys %$over ];
+        push @$res, $tmp;
+    }
+    return $self->sendJSONresponse( $req, $res );
+}
+
 # 33 - Root queries
 #      -----------
 
