@@ -5,7 +5,7 @@ use Mouse;
 use Lemonldap::NG::Common::Conf::Constants;
 use JSON qw(from_json to_json);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.6';
 
 has sessionTypes => ( is => 'rw' );
 
@@ -201,8 +201,8 @@ sub delete2F {
     return $self->sendJSONresponse( $req, { result => 1 } );
 }
 
-sub session {
-    my ( $self, $req, $id, $skey ) = @_;
+sub _session {
+    my ( $self, $raw, $req, $id, $skey ) = @_;
     my ( %h, $res );
     return $self->sendError( $req, 'Bad request', 400 ) unless ($id);
     my $mod = $self->getMod($req)
@@ -214,16 +214,18 @@ sub session {
 
     my %session = %{ $apacheSession->data };
 
-    foreach my $k ( keys %session ) {
-        $session{$k} = '**********'
-          if ( $self->hAttr =~ /\b$k\b/ );
-        $session{$k} = [ split /$self->separator/o, $session{$k} ]
-          if ( $session{$k} =~ /$self->separator/o );
+    unless ($raw) {
+        foreach my $k ( keys %session ) {
+            $session{$k} = '**********'
+              if ( $self->hAttr =~ /\b$k\b/ );
+            $session{$k} = [ split /$self->separator/o, $session{$k} ]
+              if ( $session{$k} =~ /$self->separator/o );
+        }
     }
 
     if ($skey) {
         if ( $skey =~ s/^\[(.*)\]$/$1/ ) {
-            my @sk = split /,/, $skey;
+            my @sk  = split /,/, $skey;
             my $res = {};
             $res->{$_} = $session{$_} foreach (@sk);
             return $self->sendJSONresponse( $req, $res );
@@ -237,6 +239,16 @@ sub session {
     # TODO: check for utf-8 problems
 }
 
+sub session {
+    my $self = shift;
+    return $self->_session( 0, @_ );
+}
+
+sub rawSession {
+    my $self = shift;
+    return $self->_session( 1, @_ );
+}
+
 sub getApacheSession {
     my ( $self, $mod, $id, $info, $force ) = @_;
     my $apacheSession = Lemonldap::NG::Common::Session->new( {
@@ -248,7 +260,7 @@ sub getApacheSession {
               Lemonldap::NG::Handler::PSGI::Main->tsv->{sessionCacheOptions},
             id    => $id,
             force => $force,
-            kind  => $mod->{kind},
+            ( $id ? () : ( kind => $mod->{kind} ) ),
             ( $info ? ( info => $info ) : () ),
         }
     );
@@ -264,13 +276,16 @@ sub getMod {
     my ( $self, $req ) = @_;
     my ( $s, $m );
     unless ( $s = $req->params('sessionType') ) {
-        $self->error('Session type is required');
+        $self->error($req->error('Session type is required'));
         return ();
     }
     unless ( $m = $self->sessionTypes->{$s} ) {
-        $self->error('Unknown (or unconfigured) session type');
+        $self->error($req->error('Unknown (or unconfigured) session type'));
         return ();
     }
+    if ( my $kind = $req->params('kind') ) {
+        $m->{kind} = $kind;
+    }
     return $m;
 }
 
