@@ -17,7 +17,7 @@ use JSON;
 use Lemonldap::NG::Common::Conf::Constants;
 use Lemonldap::NG::Common::PSGI::Constants;
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.6';
 
 extends 'Lemonldap::NG::Common::Conf::AccessLib',
   'Lemonldap::NG::Handler::PSGI::Router';
@@ -87,14 +87,22 @@ sub init {
         "default-src 'self' $portal;frame-ancestors 'none';form-action 'self';"
     );
 
-    $self->defaultRoute( $working[0]->defaultRoute );
+    # Avoid restricted users to access configuration by default route
+    my $defaultMod       = $self->{defaultModule} || 'conf';
+    $self->logger->debug("Default module -> $defaultMod");
+    my ($index) =
+      grep { $working[$_] =~ /::$defaultMod$/i } ( 0 .. $#working );
+    $index //= $#working;
+    $self->logger->debug("Default index -> $index");
+    $self->defaultRoute( $working[$index]->defaultRoute );
 
 # Find out more glyphicones at https://www.w3schools.com/icons/bootstrap_icons_glyphicons.asp
     my $linksIcons = {
         'conf'          => 'cog',
         'sessions'      => 'duplicate',
         'notifications' => 'bell',
-        '2ndFA'         => 'wrench'
+        '2ndFA'         => 'wrench',
+        'viewer'        => 'eye-open',
     };
 
     $self->links( [] );
@@ -132,13 +140,23 @@ sub init {
 }
 
 sub tplParams {
-    return ( VERSION => $VERSION, );
+    my ( $self, $req ) = @_;
+    my $res = $self->brwRule->( $req, $req->{userData} ) || 0;
+    return ( VERSION => $VERSION, ALLOWBROWSER => $res );
 }
 
 sub javascript {
-    my ($self) = @_;
+    my ( $self, $req ) = @_;
+    my $res       = $self->diffRule->( $req, $req->{userData} ) || 0;
+    my $impPrefix = $self->{impersonationPrefix};
+    my $ttl       = $self->{timeout} || 72000;
+
     return
-      'var formPrefix=staticPrefix+"forms/";var confPrefix=scriptname+"confs/";'
+'var formPrefix=staticPrefix+"forms/";var confPrefix=scriptname+"confs/";var viewPrefix=scriptname+"view/";'
+      . 'var allowDiff=' . "$res;"
+      . 'var impPrefix=' . "'"
+      . $impPrefix . "'" . ';'
+      . 'var sessionTTL=' . "$ttl;"
       . ( $self->links ? 'var links=' . to_json( $self->links ) . ';' : '' )
       . (
         $self->menuLinks
