@@ -9,12 +9,15 @@
 #
 package Lemonldap::NG::Portal::Main::Run;
 
-our $VERSION = '2.0.1';
+our $VERSION = '2.0.6';
 
 package Lemonldap::NG::Portal::Main;
 
 use strict;
 use URI::Escape;
+use JSON;
+
+has trOverCache => ( is => 'rw', default => sub { {} } );
 
 # List constants
 sub authProcess { qw(extractFormInfo getUser authenticate) }
@@ -52,6 +55,21 @@ sub handler {
     }
     my $res = $self->Lemonldap::NG::Common::PSGI::Router::handler($req);
 
+    # Avoid permanent loop 'Portal <-> _url' if pdata cookie is not removed
+    my $url64 = encode_base64( $req->userData->{_url}, '' )
+      if $req->userData->{_url};
+    if (    $url64
+        and !$req->pdata->{keepPdata}
+        and $req->userData->{_session_id}
+        and $req->{env}->{HTTP_COOKIE}
+        and $req->{env}->{HTTP_COOKIE} =~ /$url64/ )
+    {
+        $self->logger->debug("Force cleaning pdata");
+        $self->logger->warn("pdata cookie domain must be set")
+          unless ( $self->conf->{pdataDomain} );
+        $req->pdata( {} );
+    }
+
     # Save pdata
     if ( $sp or %{ $req->pdata } ) {
         my %v = (
@@ -59,11 +77,13 @@ sub handler {
             (
                 %{ $req->pdata }
                 ? ( value => uri_escape( JSON::to_json( $req->pdata ) ) )
-                : (
-                    value   => '',
-                    expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
-                )
-            )
+                : ( value => '', expires => 'Wed, 21 Oct 2015 00:00:00 GMT' )
+            ),
+            (
+                $self->conf->{pdataDomain}
+                ? ( domain => $self->conf->{pdataDomain}, )
+                : ()
+            ),
         );
         push @{ $res->[1] }, 'Set-Cookie', $self->cookie(%v);
     }
@@ -150,8 +170,10 @@ sub refresh {
     foreach ( keys %data ) {
         delete $data{$_} unless ( /^_/ or /^(?:startTime)$/ );
     }
-    $req->steps(
-        [
+    $data{_updateTime} = strftime( "%Y%m%d%H%M%S", localtime() );
+    $self->logger->debug(
+        "Set session $req->{id} _updateTime with $data{_updateTime}");
+    $req->steps( [
             'getUser',
             @{ $self->betweenAuthAndData },
             'setAuthSessionInfo',
@@ -161,6 +183,7 @@ sub refresh {
             'setLocalGroups',
             sub {
                 $req->sessionInfo->{$_} = $data{$_} foreach ( keys %data );
+                $req->refresh(1);
                 return PE_OK;
             },
             'store',
@@ -170,6 +193,7 @@ sub refresh {
     if ($res) {
         $req->info(
             $self->loadTemplate(
+                $req,
                 'simpleInfo', params => { trspan => 'rightsReloadNeedsLogout' }
             )
         );
@@ -184,8 +208,9 @@ sub logout {
     return $self->do(
         $req,
         [
-            'controlUrl', @{ $self->beforeLogout },
-            'authLogout', 'deleteSession'
+            'importHandlerData',      'controlUrl',
+            @{ $self->beforeLogout }, 'authLogout',
+            'deleteSession'
         ]
     );
 }
@@ -210,6 +235,12 @@ sub do {
     if ( $err == PE_SENDRESPONSE ) {
         return $req->response;
     }
+
+    # Remove userData if authentication fails
+    if ( $err == PE_BADCREDENTIALS ) {
+        $req->userData( {} );
+    }
+
     if ( !$self->conf->{noAjaxHook} and $req->wantJSON ) {
         $self->logger->debug('Processing to JSON response');
         if ( ( $err > 0 and !$req->id ) or $err eq PE_SESSIONNOTGRANTED ) {
@@ -219,7 +250,7 @@ sub do {
                 [qq'{"result":0,"error":$err}']
             ];
         }
-        elsif ( $err > 0 and $err != PE_PASSWORD_OK ) {
+        elsif ( $err > 0 and $err != PE_PASSWORD_OK and $err != PE_LOGOUT_OK ) {
             return $self->sendJSONresponse(
                 $req,
                 { result => 0, error => $err },
@@ -231,21 +262,29 @@ sub do {
                 $req,
                 {
                     result => 1,
-                    code   => $err
+                    error  => $err,
+                    id     => $req->id,
+                    (
+                        $req->sessionInfo->{_httpSession}
+                        ? ( id_http => $req->sessionInfo->{_httpSession} )
+                        : ()
+                    )
                 }
             );
         }
     }
     else {
         if (
-                $err
-            and $err != PE_LOGOUT_OK
-            and (
-                $err != PE_REDIRECT
-                or (    $err == PE_REDIRECT
-                    and $req->data->{redirectFormMethod}
-                    and $req->data->{redirectFormMethod} eq 'post' )
-                or $req->info
+            $req->info
+            or (
+                    $err
+                and $err != PE_LOGOUT_OK
+                and (
+                    $err != PE_REDIRECT
+                    or (    $err == PE_REDIRECT
+                        and $req->data->{redirectFormMethod}
+                        and $req->data->{redirectFormMethod} eq 'post' )
+                )
             )
           )
         {
@@ -306,8 +345,7 @@ sub autoRedirect {
             $req->data->{redirectFormMethod} = "get";
         }
         else {
-            return [ 302,
-                [ Location => $req->{urldc}, @{ $req->respHeaders } ], [] ];
+            return [ 302, [ Location => $req->{urldc}, $req->spliceHdrs ], [] ];
         }
     }
     my ( $tpl, $prms ) = $self->display($req);
@@ -319,7 +357,7 @@ sub autoRedirect {
 # If $id is set to undef or if $args{force} is true, return a new session.
 sub getApacheSession {
     my ( $self, $id, %args ) = @_;
-    $args{kind} ||= "SSO";
+    $args{kind} //= "SSO";
     if ($id) {
         $self->logger->debug("Try to get $args{kind} session $id");
     }
@@ -327,8 +365,7 @@ sub getApacheSession {
         $self->logger->debug("Try to get a new $args{kind} session");
     }
 
-    my $as = Lemonldap::NG::Common::Session->new(
-        {
+    my $as = Lemonldap::NG::Common::Session->new( {
             storageModule        => $self->conf->{globalStorage},
             storageModuleOptions => $self->conf->{globalStorageOptions},
             cacheModule          => $self->conf->{localSessionStorage},
@@ -388,15 +425,14 @@ sub getApacheSession {
 sub getPersistentSession {
     my ( $self, $uid, $info ) = @_;
 
-    return unless defined $uid;
+    return unless ( defined $uid and !$self->conf->{disablePersistentStorage} );
 
     # Compute persistent identifier
     my $pid = $self->_md5hash($uid);
 
     $info->{_session_uid} = $uid;
 
-    my $ps = Lemonldap::NG::Common::Session->new(
-        {
+    my $ps = Lemonldap::NG::Common::Session->new( {
             storageModule        => $self->conf->{persistentStorage},
             storageModuleOptions => $self->conf->{persistentStorageOptions},
             id                   => $pid,
@@ -436,7 +472,11 @@ sub updatePersistentSession {
     my ( $self, $req, $infos, $uid, $id ) = @_;
 
     # Return if no infos to update
-    return () unless ( ref $infos eq 'HASH' and %$infos );
+    return ()
+      unless ( ref $infos eq 'HASH'
+        and %$infos
+        and !$self->conf->{disablePersistentStorage} );
+
     $uid ||= $req->{sessionInfo}->{ $self->conf->{whatToTrace} }
       || $req->userData->{ $self->conf->{whatToTrace} };
     $self->logger->debug("Found 'whatToTrace' -> $uid");
@@ -481,8 +521,8 @@ sub updateSession {
         ## sessionInfo updated if $id defined : quite strange !!
         ## See https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues/430
         foreach ( keys %$infos ) {
-            $self->logger->debug(
-                "Update sessionInfo $_ with " . $infos->{$_} );
+            $self->logger->debug("Update sessionInfo $_");
+            $self->_dump( $infos->{$_} );
             $req->{sessionInfo}->{$_} = $self->HANDLER->data->{$_} =
               $infos->{$_};
         }
@@ -553,7 +593,10 @@ sub _deleteSession {
 
     # Log
     my $user = $req->{sessionInfo}->{ $self->conf->{whatToTrace} };
-    $self->userLogger->notice("User $user has been disconnected") if $user;
+    my $mod  = $req->{sessionInfo}->{_auth};
+    $self->userLogger->notice(
+"User $user has been disconnected from $mod ($req->{sessionInfo}->{ipAddr})"
+    ) if $user;
 
     return $session->error ? 0 : 1;
 }
@@ -607,7 +650,7 @@ sub autoPost {
 
 # Add element into $self->{portalHiddenFormValues}, those values could be
 # used to hide values into HTML form.
-# @param fieldname The field name which will contain the correponding value
+# @param fieldname The field name which will contain the corresponding value
 # @param value The associated value
 # @param prefix Prefix of the field key
 # @param base64 Encode value in base64
@@ -624,7 +667,7 @@ sub setHiddenFormValue {
     if ( defined $val or !( $val & ~$val ) ) {
         $key = $prefix . $key;
 
-        #$val =~ s/\+/%2B/g;
+        $val = encode_base64($val) if $base64;
         $req->{portalHiddenFormValues}->{$key} = $val;
         $self->logger->debug("Store $val in hidden key $key");
     }
@@ -720,20 +763,69 @@ sub cookie {
 
 sub _dump {
     my ( $self, $variable ) = @_;
-    require Data::Dumper;
-    $Data::Dumper::Indent  = 0;
-    $Data::Dumper::Useperl = 1;
-    $self->logger->debug( "Dump: " . Data::Dumper::Dumper($variable) );
+    if ( $self->conf->{logLevel} eq 'debug' ) {
+        require Data::Dumper;
+        $Data::Dumper::Indent  = 0;
+        $Data::Dumper::Useperl = 1;
+        $self->logger->debug( "Dump: " . Data::Dumper::Dumper($variable) );
+    }
     return;
 }
 
 sub sendHtml {
     my ( $self, $req, $template, %args ) = @_;
-    $args{params}->{TROVER} = $self->trOver;
+
+    my $templateDir = $self->conf->{templateDir} . '/' . $self->getSkin($req);
+    $self->templateDir( [ $templateDir, @{ $self->templateDir } ] );
+
+    # Check template
+    $args{templateDir} = $templateDir;
+    my $tmpl = $args{templateDir} . "/$template.tpl";
+    unless ( -f $tmpl ) {
+        $self->logger->debug("Template $tmpl not found");
+        $args{templateDir} = $self->conf->{templateDir} . '/bootstrap';
+        $tmpl = $args{templateDir} . "/$template.tpl";
+        $self->logger->debug("-> Trying to load $tmpl");
+    }
+
+    # Override messages
+    my $trOverMessages = JSON::from_json( $self->trOver );
+
+    unless ( $self->trOverCache->{$templateDir} ) {
+        opendir( DIR, $templateDir );
+        my @langfiles = grep( /\.json$/, readdir(DIR) );
+        close(DIR);
+
+        foreach my $file (@langfiles) {
+            my ($lang) = ( $file =~ /^(\w+)\.json/ );
+            $self->logger->debug("Use $file to override messages");
+            if ( open my $json, "<", $templateDir . "/" . $file ) {
+                local $/ = undef;
+                $trOverMessages->{$lang} = JSON::from_json(<$json>);
+            }
+            else {
+                $self->logger->error("Unable to read $file");
+            }
+        }
+
+        $self->trOverCache->{$templateDir} = JSON::to_json($trOverMessages);
+    }
+    $args{params}->{TROVER} = $self->trOverCache->{$templateDir};
+
     my $res = $self->SUPER::sendHtml( $req, $template, %args );
     push @{ $res->[1] },
       'X-XSS-Protection'       => '1; mode=block',
-      'X-Content-Type-Options' => 'nosniff';
+      'X-Content-Type-Options' => 'nosniff',
+      'Cache-Control' => 'no-cache, no-store, must-revalidate',    # HTTP 1.1
+      'Pragma'        => 'no-cache',                               # HTTP 1.0
+      'Expires'       => '0';                                      # Proxies
+
+    if ( $self->conf->{corsEnabled} ) {
+        my @cors = split /;/, $self->cors;
+        push @{ $res->[1] }, @cors;
+        $self->logger->debug('Apply following CORS policy :');
+        $self->logger->debug(" $_") for @cors;
+    }
 
     # Set authorized URL for POST
     my $csp = $self->csp . "form-action " . $self->conf->{cspFormAction};
@@ -782,7 +874,7 @@ sub sendHtml {
           ( $req->info =~ /<iframe.*?src="(.*?)"/sg );
     }
     if (@url) {
-        $csp .= join( ' ', 'child-src', @url ) . ';';
+        $csp .= join( ' ', 'child-src', @url, "'self'" ) . ';';
     }
 
     # Set CSP header
@@ -793,13 +885,16 @@ sub sendHtml {
 
 sub sendCss {
     my ( $self, $req ) = @_;
-    my $s =
-        'html,body{background:url("'
-      . $self->staticPrefix
-      . '/common/backgrounds/'
-      . $self->conf->{portalSkinBackground}
-      . '") no-repeat center fixed;'
-      . 'background-size:cover;}';
+    my $s = '/* LL::NG Portal CSS */';
+    if ( $self->conf->{portalSkinBackground} ) {
+        $s .=
+            'html,body{background:url("'
+          . $self->staticPrefix
+          . '/common/backgrounds/'
+          . $self->conf->{portalSkinBackground}
+          . '") no-repeat center fixed;'
+          . 'background-size:cover;}';
+    }
     return [
         200,
         [
@@ -812,11 +907,20 @@ sub sendCss {
 }
 
 sub lmError {
-    my ( $self, $req ) = @_;
-    my $httpError = $req->param('code');
+    my ( $self, $req, $error ) = @_;
+    my $httpError = $req->param('code') || $error;
 
     # Check URL
     $self->controlUrl($req);
+    $req->pdata( {} ) unless ( $httpError == 404 );
+
+    if ( $req->wantJSON ) {
+        return $self->sendJSONresponse(
+            $req,
+            { error => $httpError, result => 0 },
+            code => $httpError
+        );
+    }
 
     my %templateParams = (
         MAIN_LOGO  => $self->conf->{portalMainLogo},
@@ -835,24 +939,37 @@ sub rebuildCookies {
     my ( $self, $req ) = @_;
     my @tmp;
     for ( my $i = 0 ; $i < @{ $req->{respHeaders} } ; $i += 2 ) {
-        push @tmp, $req->respHeaders->[0], $req->respHeaders->[1]
-          unless ( $req->respHeaders->[0] eq 'Set-Cookie' );
+        push @tmp, $req->respHeaders->[$i], $req->respHeaders->[ $i + 1 ]
+          unless ( $req->respHeaders->[$i] eq 'Set-Cookie' );
     }
     $req->{respHeaders} = \@tmp;
     $self->buildCookie($req);
 }
 
 sub tplParams {
-    my $portalPath = $_[0]->conf->{portal};
+    my ( $self, $req ) = @_;
+    my %templateParams;
+
+    my $portalPath = $self->conf->{portal};
     $portalPath =~ s#^https?://[^/]+/?#/#;
     $portalPath =~ s#[^/]+\.fcgi$##;
+
+    for my $session_key ( keys %{ $req->{sessionInfo} } ) {
+        $templateParams{ "session_" . $session_key } =
+          $req->{sessionInfo}->{$session_key};
+    }
+
+    for my $env_key ( keys %{ $req->env } ) {
+        $templateParams{ "env_" . $env_key } = $req->env->{$env_key};
+    }
+
     return (
-        SKIN       => $_[0]->getSkin( $_[1] ),
-        PORTAL_URL => $_[0]->conf->{portal},
+        SKIN       => $self->getSkin($req),
+        PORTAL_URL => $self->conf->{portal},
         SKIN_PATH  => $portalPath . "skins",
-        ANTIFRAME  => $_[0]->conf->{portalAntiFrame},
-        SKIN_BG    => $_[0]->conf->{portalSkinBackground},
-        ( $_[0]->customParameters ? ( %{ $_[0]->customParameters } ) : () ),
+        SKIN_BG    => $self->conf->{portalSkinBackground},
+        ( $self->customParameters ? ( %{ $self->customParameters } ) : () ),
+        %templateParams
     );
 }
 
@@ -861,6 +978,36 @@ sub registerLogin {
     return
       unless ( $self->conf->{loginHistoryEnabled}
         and defined $req->authResult );
+
+    # Check old login history
+    if ( $req->sessionInfo->{loginHistory} ) {
+
+        if ( !$req->sessionInfo->{_loginHistory} ) {
+            $self->logger->debug("Restore old login history");
+
+            # Restore success login
+            $req->sessionInfo->{_loginHistory}->{successLogin} =
+              $req->sessionInfo->{loginHistory}->{successLogin};
+
+            # Restore failed login, with generic error
+            if ( $req->sessionInfo->{loginHistory}->{failedLogin} ) {
+                $self->logger->debug("Restore old failed logins");
+                $req->sessionInfo->{_loginHistory}->{failedLogin} = [];
+                foreach (
+                    @{ $req->sessionInfo->{loginHistory}->{failedLogin} } )
+                {
+                    $self->logger->debug(
+                        "Replace old failed login error " . $_->{error} );
+                    $_->{error} = 5;
+                    push @{ $req->sessionInfo->{_loginHistory}->{failedLogin} },
+                      $_;
+                }
+            }
+        }
+        $self->updatePersistentSession( $req, { 'loginHistory' => undef } );
+        delete $req->sessionInfo->{loginHistory};
+    }
+
     my $history = $req->sessionInfo->{_loginHistory} ||= {};
     my $type = ( $req->authResult > 0 ? 'failed' : 'success' ) . 'Login';
     $history->{$type} ||= [];
@@ -902,24 +1049,45 @@ sub _sumUpSession {
     return $res;
 }
 
+sub corsPreflight {
+    my ( $self, $req ) = @_;
+    my @headers;
+    if ( $self->conf->{corsEnabled} ) {
+        my @cors = split /;/, $self->cors;
+        push @headers, @cors;
+        $self->logger->debug('Apply following CORS policy :');
+        $self->logger->debug(" $_") for @cors;
+    }
+    return [ 204, \@headers, [] ];
+}
+
+sub sendJSONresponse {
+    my ( $self, $req, $j, %args ) = @_;
+    my $res = Lemonldap::NG::Common::PSGI::sendJSONresponse(@_);
+    if ( $self->conf->{corsEnabled} ) {
+        my @cors = split /;/, $self->cors;
+        push @{ $res->[1] }, @cors;
+        $self->logger->debug('Apply following CORS policy :');
+        $self->logger->debug(" $_") for @cors;
+    }
+    return $res;
+}
+
 # Temlate loader
 sub loadTemplate {
-    my ( $self, $name, %prm ) = @_;
+    my ( $self, $req, $name, %prm ) = @_;
     $name .= '.tpl';
-    my $file =
-        $self->conf->{templateDir} . '/'
-      . $self->conf->{portalSkin} . '/'
-      . $name;
-    $file = $self->conf->{templateDir} . '/common/' . $name
-      unless ( -e $file );
-    unless ( -e $file ) {
-        die "Unable to find $name in $self->conf->{templateDir}";
-    }
     my $tpl = HTML::Template->new(
-        filename               => $file,
+        filename => $name,
+        path     => [
+            $self->conf->{templateDir} . '/' . $self->getSkin($req),
+            $self->conf->{templateDir} . '/bootstrap/',
+            $self->conf->{templateDir} . '/common/'
+        ],
+        search_path_on_include => 1,
         die_on_bad_params      => 0,
         die_on_missing_include => 1,
-        cache                  => 1,
+        cache                  => ( defined $prm{cache} ? $prm{cache} : 1 ),
         global_vars            => 0,
         ( $prm{filter} ? ( filter => $prm{filter} ) : () ),
     );
